@@ -4,7 +4,7 @@ use warnings;
 use strict;
 use 5.010001;
 
-our $VERSION = '1.112';
+our $VERSION = '1.113';
 use Exporter 'import';
 our @EXPORT_OK = qw( choose );
 
@@ -81,6 +81,68 @@ sub __set_defaults {
 }
 
 
+sub _next_release__validate_options {
+    my ( $self, $opt ) = @_;
+    return if ! defined $opt;
+    my $valid = {
+        beep            => '[ 0 1 ]',
+        clear_screen    => '[ 0 1 ]',
+        default         => '[ 0-9 ]+',
+        empty           => '',
+        hide_cursor     => '[ 0 1 ]',
+        index           => '[ 0 1 ]',
+        justify         => '[ 0 1 2 ]',
+        keep            => '[ 1-9 ][ 0-9 ]*',
+        layout          => '[ 0 1 2 3 ]',
+        lf              => 'ARRAY',
+        ll              => '[ 1-9 ][ 0-9 ]*',
+        limit           => '[ 1-9 ][ 0-9 ]*',
+        max_height      => '[ 1-9 ][ 0-9 ]*',
+        max_width       => '[ 1-9 ][ 0-9 ]*',
+        mouse           => '[ 0 1 2 3 4 ]',
+        no_spacebar     => 'ARRAY',
+        order           => '[ 0 1 ]',
+        pad             => '[ 0-9 ]+',
+        pad_one_row     => '[ 0-9 ]+',
+        page            => '[ 0 1 ]',
+        prompt          => '',
+        undef           => '',
+    };
+    my $sub =  ( caller( 1 ) )[3];
+    $sub =~ s/^.+::([^:]+)\z/$1/;
+    for my $key ( keys %$opt ) {
+        if ( ! exists $valid->{$key} ) {
+            croak "$sub: '$key' is not a valid option name";
+        }
+        next if ! defined $opt->{$key};
+        if ( $key eq 'lf' ) {
+            if ( ! ( ref $opt->{$key} eq 'ARRAY' && @{$opt->{$key}} <= 2 ) ) {
+                croak "$sub: option '$key' : the passed value has to be an ARRAY reference.";
+            }
+            no warnings 'uninitialized';
+            for ( @{$opt->{$key}} ) {
+                /^[0-9]+\z/ or croak "$sub: option '$key' : $_ is an invalid array element";
+            }
+        }
+        elsif ( $key eq 'no_spacebar' ) {
+            if ( ref $opt->{$key} ne 'ARRAY' ) {
+                croak "$sub: option '$key' : the passed value has to be an ARRAY reference.";
+            }
+            no warnings 'uninitialized';
+            for ( @{$opt->{$key}} ) {
+                /^[0-9]+\z/ or croak "$sub: option '$key' : $_ is an invalid array element";
+            }
+        }
+        elsif ( $valid->{$key} eq '' && ref $opt->{$key} ne '' ) {
+            croak "$sub: option '$key' : references are not valid values.";
+        }
+        elsif ( length $valid->{$key} && $opt->{$key} !~ m/^$valid->{$key}\z/x ) {
+            croak "$sub: option '$key' : '$opt->{$key}' is not a valid value.";
+        }
+        $self->{$key} = $opt->{$key};
+    }
+}
+
 sub __validate_options {
     my ( $self, $opt ) = @_;
     return if ! defined $opt;
@@ -169,23 +231,19 @@ sub __validate_options {
 
 sub __init_term {
     my ( $self ) = @_;
-    $self->{mouse} = $self->{plugin}->__set_mode( $self->{mouse} );
-    print HIDE_CURSOR if $self->{hide_cursor};
+    $self->{mouse} = $self->{plugin}->__set_mode( $self->{mouse}, $self->{hide_cursor} );
 }
 
 
 sub __reset_term {
     my ( $self, $from_choose ) = @_;
     if ( $from_choose ) {
-        print CR, UP x ( $self->{i_row} + $self->{nr_prompt_lines} );
-        print CLEAR_TO_END_OF_SCREEN;
+        print CR;
+        $self->{plugin}->__up( $self->{i_row} + $self->{nr_prompt_lines} );
+        $self->{plugin}->__clear_to_end_of_screen();  ###
     }
-    print RESET;
     if ( defined $self->{plugin} ) {
-        $self->{plugin}->__reset_mode( $self->{mouse} );
-    }
-    if ( $self->{hide_cursor} ) {
-        print SHOW_CURSOR;
+        $self->{plugin}->__reset_mode( $self->{mouse}, $self->{hide_cursor} );
     }
     if ( defined $self->{backup_opt} ) {
         my $backup_opt = delete $self->{backup_opt};
@@ -217,7 +275,7 @@ sub config {
 
 sub choose {
     if ( ref $_[0] ne 'Term::Choose' ) {
-        return Term::Choose->new( $_[1] )->choose( $_[0] );
+        return Term::Choose->new()->choose( @_ );
     }
     my $self = shift;
     my ( $orig_list_ref, $opt ) = @_;
@@ -261,8 +319,9 @@ sub choose {
         my ( $new_width, $new_height ) = $self->{plugin}->__get_term_size();
         if ( $new_width != $self->{term_width} || $new_height != $self->{term_height} ) {
             $self->{list} = $self->__copy_orig_list();
-            print CR, UP x ( $self->{i_row} + $self->{nr_prompt_lines} );
-            print CLEAR_TO_END_OF_SCREEN;
+            print CR;
+            $self->{plugin}->__up( $self->{i_row} + $self->{nr_prompt_lines} );
+            $self->{plugin}->__clear_to_end_of_screen();
             $self->__write_first_screen();
             next;
         }
@@ -738,10 +797,10 @@ sub __write_first_screen {
     $self->{i_col}      = 0;
     $self->{pos}        = [ 0, 0 ];
     $self->__set_default_cell() if defined $self->{default} && $self->{default} <= $#{$self->{list}};
-    print CLEAR_SCREEN if $self->{clear_screen};
+    $self->{plugin}->__clear_screen() if $self->{clear_screen};
     print $self->{prompt_copy} if $self->{prompt} ne '';
     $self->__wr_screen();
-    $self->{plugin}->__term_cursor_position() if $self->{mouse};
+    $self->{plugin}->__get_cursor_position() if $self->{mouse};
     $self->{cursor_row} = $self->{i_row};
 }
 
@@ -946,15 +1005,18 @@ sub __goto {
         $self->{i_col} = 0;
     }
     elsif ( $newrow < $self->{i_row} ) {
-        print UP x ( $self->{i_row} - $newrow );
+        #print UP x ( $self->{i_row} - $newrow );
+        $self->{plugin}->__up( $self->{i_row} - $newrow );
         $self->{i_row} -= ( $self->{i_row} - $newrow );
     }
     if ( $newcol > $self->{i_col} ) {
-        print RIGHT x ( $newcol - $self->{i_col} );
+        #print RIGHT x ( $newcol - $self->{i_col} );
+        $self->{plugin}->__right( $newcol - $self->{i_col} );
         $self->{i_col} += ( $newcol - $self->{i_col} );
     }
     elsif ( $newcol < $self->{i_col} ) {
-        print LEFT x ( $self->{i_col} - $newcol );
+        #print LEFT x ( $self->{i_col} - $newcol );
+        $self->{plugin}->__left( $self->{i_col} - $newcol );
         $self->{i_col} -= ( $self->{i_col} - $newcol );
     }
 }
@@ -963,7 +1025,7 @@ sub __goto {
 sub __wr_screen {
     my ( $self ) = @_;
     $self->__goto( 0, 0 );
-    print CLEAR_TO_END_OF_SCREEN;
+    $self->{plugin}->__clear_to_end_of_screen();
     if ( $self->{page} && $self->{pp} > 1 ) {
         $self->__goto( $self->{avail_height_idx} + $self->{tail}, 0 );
         if ( $self->{pp_printf_type} == 0 ) {
@@ -996,20 +1058,20 @@ sub __wr_cell {
             }
         }
         $self->__goto( $row - $self->{row_on_top}, $lngth );
-        print BOLD, UNDERLINE if $self->{marked}[$row][$col];
-        print REVERSE         if $row == $self->{pos}[ROW] && $col == $self->{pos}[COL];
+        $self->{plugin}->__bold_underline() if $self->{marked}[$row][$col];
+        $self->{plugin}->__reverse()        if $row == $self->{pos}[ROW] && $col == $self->{pos}[COL];
         print $self->{list}[$self->{rc2idx}[$row][$col]];
         my $gcs_element = Unicode::GCString->new( $self->{list}[$self->{rc2idx}[$row][$col]] );
         $self->{i_col} += $gcs_element->columns();
     }
     else {
         $self->__goto( $row - $self->{row_on_top}, $col * $self->{col_width} );
-        print BOLD, UNDERLINE if $self->{marked}[$row][$col];
-        print REVERSE         if $row == $self->{pos}[ROW] && $col == $self->{pos}[COL];
+        $self->{plugin}->__bold_underline() if $self->{marked}[$row][$col];
+        $self->{plugin}->__reverse()        if $row == $self->{pos}[ROW] && $col == $self->{pos}[COL];
         print $self->__unicode_sprintf( $self->{rc2idx}[$row][$col] );
         $self->{i_col} += $self->{length_longest};
     }
-    print RESET if $self->{marked}[$row][$col] || $row == $self->{pos}[ROW] && $col == $self->{pos}[COL];
+    $self->{plugin}->__reset() if $self->{marked}[$row][$col] || $row == $self->{pos}[ROW] && $col == $self->{pos}[COL];
 }
 
 
@@ -1138,7 +1200,7 @@ Term::Choose - Choose items from a list interactively.
 
 =head1 VERSION
 
-Version 1.112
+Version 1.113
 
 =cut
 
@@ -1178,6 +1240,13 @@ Object-oriented interface:
     my $stopp = Term::Choose->new( { prompt => '' } );
     $stopp->choose( [ 'Press ENTER to continue' ] );               # no choice
 
+=head1 ANNOUNCEMENTS
+
+Passing invalid options will become fatal with the next release!
+
+MSWin32: from the next release on L<Win32::Console::ANSI> will be removed! L<Win32::Console> will be used then to
+emulate the escape sequences.
+
 =head1 DESCRIPTION
 
 Choose interactively from a list of items.
@@ -1671,6 +1740,8 @@ default: "<undef>"
 
 =head2 carp
 
+Passing invalid options will become fatal with the next release!
+
 =over
 
 =item * If the array referred by the first argument is empty C<choose> warns and returns C<undef> respective an empty
@@ -1717,7 +1788,7 @@ L<Win32::Console>
 
 =item
 
-L<Win32::Console::ANSI>
+L<Win32::Console::ANSI> (removed with the next release)
 
 =back
 
@@ -1743,6 +1814,8 @@ For a correct output it is required an appropriate encoding layer for STDOUT mat
 
 =head3 MSWin32 operating systems
 
+C<Win32::Console::ANSI> will be removed from the next release on so the following will become obsolete!
+
 C<Term::Choose> uses L<Win32::Console::ANSI> if the operating system is MSWin32.
 
 C<Win32::Console::ANSI> enables the Windows own codepage conversion globally. C<Term::Choose> disables this codepage
@@ -1805,6 +1878,8 @@ If the option "clear_screen" is enabled:
 
 If the OS is MSWin32 the L<Win32::Console::ANSI> module is used to understand these escape sequences.
 
+C<Win32::Console::ANSI> will be replaced by C<Win32::Console> from the next release on to emulate the escape sequences.
+
 If a I<mouse> mode is enabled
 
     "\e[6n"    Get Cursor Position (Device Status Report)
