@@ -5,7 +5,7 @@ use warnings;
 use strict;
 use 5.010001;
 
-our $VERSION = '1.112';
+our $VERSION = '1.113';
 
 use Term::ReadKey qw( GetTerminalSize ReadKey ReadMode );
 
@@ -139,7 +139,7 @@ sub __mouse_event_to_button {
 
 
 sub __set_mode {
-    my ( $self, $mouse ) = @_;
+    my ( $self, $mouse, $hide_cursor ) = @_;
     if ( $mouse ) {
         if ( $mouse == 3 ) {
             my $return = binmode STDIN, ':utf8';
@@ -178,18 +178,21 @@ sub __set_mode {
         }
     }
     Term::ReadKey::ReadMode( 'ultra-raw' );
+    print HIDE_CURSOR if $hide_cursor;
     return $mouse;
 };
 
 
 sub __reset_mode {
-    my ( $self, $mouse ) = @_;
+    my ( $self, $mouse, $hide_cursor ) = @_;
+    print SHOW_CURSOR if $hide_cursor;
     if ( $mouse ) {
         binmode STDIN, ':encoding(UTF-8)' or warn "binmode STDIN, :encoding(UTF-8): $!\n";
         print UNSET_EXT_MODE_MOUSE_1005     if $mouse == 3;
         print UNSET_SGR_EXT_MODE_MOUSE_1006 if $mouse == 4;
         print UNSET_ANY_EVENT_MOUSE_1003;
     }
+    $self->__reset();
     Term::ReadKey::ReadMode( 'restore' );
 }
 
@@ -200,7 +203,7 @@ sub __get_term_size {
 }
 
 
-sub __term_cursor_position {
+sub __get_cursor_position {
     my ( $self ) = @_;
     #$self->{abs_cursor_x} = 0; # unused
     $self->{abs_cursor_y} = 0;
@@ -208,8 +211,53 @@ sub __term_cursor_position {
 }
 
 
+sub __clear_screen {
+    #my ( $self ) = @_;
+    print "\e[2J\e[1;1H";
+}
 
 
+sub __clear_to_end_of_screen {
+    #my ( $self ) = @_;
+    print "\e[0J";
+}
+
+
+sub __bold_underline {
+    #my ( $self ) = @_;
+    print "\e[1m\e[4m";
+}
+
+
+sub __reverse {
+    #my ( $self ) = @_;
+    print "\e[7m";
+}
+
+
+sub __reset {
+    #my ( $self ) = @_;
+    print "\e[0m";
+}
+
+
+sub __up {
+    #my ( $self ) = @_;
+    print "\e[${_[1]}A";
+}
+
+
+sub __left {
+    #my ( $self ) = @_;
+    print "\e[${_[1]}D";
+}
+
+
+sub __right {
+    #my ( $self ) = @_;
+    print "\e[${_[1]}C";
+}
+
 
 1;
 
@@ -225,7 +273,7 @@ Term::Choose::Linux - Plugin for Term::Choose.
 
 =head1 VERSION
 
-Version 1.112
+Version 1.113
 
 =head1 SYNOPSIS
 
