@@ -5,10 +5,11 @@ use warnings;
 use strict;
 use 5.010001;
 
-our $VERSION = '1.112';
+our $VERSION = '1.113';
 
-use Win32::Console       qw( STD_INPUT_HANDLE ENABLE_MOUSE_INPUT ENABLE_PROCESSED_INPUT
-                             RIGHT_ALT_PRESSED LEFT_ALT_PRESSED RIGHT_CTRL_PRESSED LEFT_CTRL_PRESSED SHIFT_PRESSED );
+use Win32::Console       qw( STD_INPUT_HANDLE ENABLE_MOUSE_INPUT ENABLE_PROCESSED_INPUT STD_OUTPUT_HANDLE
+                             RIGHT_ALT_PRESSED LEFT_ALT_PRESSED RIGHT_CTRL_PRESSED LEFT_CTRL_PRESSED SHIFT_PRESSED
+                             FOREGROUND_INTENSITY BACKGROUND_INTENSITY );
 use Win32::Console::ANSI qw( :func );
 
 use Term::Choose::Constants qw( :win32 );
@@ -102,17 +103,23 @@ sub __get_key_OS {
 
 
 sub __set_mode {
-    my ( $self, $mouse ) = @_;
+    my ( $self, $mouse, $hide_cursor ) = @_;
     $self->{input} = Win32::Console->new( STD_INPUT_HANDLE );
     $self->{old_in_mode} = $self->{input}->Mode();
     $self->{input}->Mode( !ENABLE_PROCESSED_INPUT )                    if ! $mouse;
     $self->{input}->Mode( !ENABLE_PROCESSED_INPUT|ENABLE_MOUSE_INPUT ) if   $mouse;
+    $self->{output} = Win32::Console->new( STD_OUTPUT_HANDLE );
+    $self->{def_attr} = $self->{output}->Attr();
+    $self->{fg_color} = $self->{def_attr} & 0x7;
+    $self->{bg_color} = $self->{def_attr} & 0x70;
+    $self->{inverse}  = ( $self->{bg_color} >> 4 ) | ( $self->{fg_color} << 4 );
+    $self->{output}->Cursor( -1, -1, -1, 0 ) if $hide_cursor;
     return $mouse;
 }
 
 
 sub __reset_mode {
-    my ( $self, $mouse ) = @_;  # no use for $mouse on win32
+    my ( $self, $mouse, $hide_cursor ) = @_;  # no use for $mouse on win32
     if ( defined $self->{input} ) {
         if ( defined $self->{old_in_mode} ) {
             $self->{input}->Mode( $self->{old_in_mode} );
@@ -123,6 +130,12 @@ sub __reset_mode {
         delete $self->{input}{handle};
         #
     }
+    if ( defined $self->{output} ) {
+        $self->__reset;
+        $self->{output}->Cursor( -1, -1, -1, 1 ) if $hide_cursor;
+        #$self->{output}->Free();
+        delete $self->{output}{handle}; # ?
+    }
 }
 
 
@@ -133,16 +146,69 @@ sub __get_term_size {
 }
 
 
-sub __term_cursor_position {
+sub __get_cursor_position {
     my ( $self ) = @_;
-    ( $self->{abs_cursor_x}, $self->{abs_cursor_y} ) = Cursor();
-    #$self->{abs_cursor_x}--; # unused
-    $self->{abs_cursor_y}--;
+    ( $self->{abs_cursor_x}, $self->{abs_cursor_y} ) = $self->{output}->Cursor();
+}
+
+sub __set_cursor_position {
+    my ( $self, $col, $row ) = @_;
+    $self->{output}->Cursor( $col, $row );
 }
 
 
+sub __clear_screen {
+    my ( $self ) = @_;
+    $self->{output}->cls( $self->{def_attr} );
+}
 
 
+sub __clear_to_end_of_screen {
+    my ( $self ) = @_;
+    my ( $width, $height ) = $self->{output}->Size(); ###
+    $self->__get_cursor_position();
+    $self->{output}->FillAttr(
+            $self->{bg_color} | $self->{bg_color},
+            $width * $height,
+            $self->{abs_cursor_x}, $self->{abs_cursor_y} );
+}
+
+
+sub __bold_underline {
+    my ( $self ) = @_;
+    $self->{output}->Attr( $self->{def_attr} | FOREGROUND_INTENSITY | BACKGROUND_INTENSITY  );
+}
+
+
+sub __reverse {
+    my ( $self ) = @_;
+    $self->{output}->Attr( $self->{inverse} );
+}
+
+
+sub __reset {
+    my ( $self ) = @_;
+    $self->{output}->Attr( $self->{def_attr} );
+}
+
+
+sub __up {
+    #my ( $self, $rows_up ) = @_;
+    my ( $col, $row ) = $_[0]->__get_cursor_position;
+    $_[0]->__set_cursor_position( $col, $row - $_[1] );
+}
+
+sub __left {
+    #my ( $self, $cols_left ) = @_;
+    my ( $col, $row ) = $_[0]->__get_cursor_position;
+    $_[0]->__set_cursor_position( $col - $_[1], $row );
+}
+
+sub __right {
+    #my ( $self, $cols_right ) = @_;
+    my ( $col, $row ) = $_[0]->__get_cursor_position;
+    $_[0]->__set_cursor_position( $col + $_[1], $row );
+}
 
 1;
 
@@ -158,7 +224,7 @@ Term::Choose::Win32 - Plugin for Term::Choose.
 
 =head1 VERSION
 
-Version 1.112
+Version 1.113
 
 =head1 DESCRIPTION
 
